/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.command;

import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;

import de.ozgcloud.command.Command;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.grpc.command.GrpcCommand;

@Mapper(uses = { CommandBodyMapper.class, GrpcObjectMapper.class }, //
		nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE, //
		nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS, //
		collectionMappingStrategy = CollectionMappingStrategy.ADDER_PREFERRED)
interface GrpcCommandMapper {

	@Mapping(target = "redirectRequest", ignore = true)
	@Mapping(target = "orderStringBytes", ignore = true)
	@Mapping(target = "createdResourceBytes", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeBody", ignore = true)
	@Mapping(target = "mergeBodyObj", ignore = true)
	@Mapping(target = "mergeRedirectRequest", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "createdAtBytes", ignore = true)
	@Mapping(target = "createdByBytes", ignore = true)
	@Mapping(target = "createdByNameBytes", ignore = true)
	@Mapping(target = "errorMessageBytes", ignore = true)
	@Mapping(target = "finishedAtBytes", ignore = true)
	@Mapping(target = "idBytes", ignore = true)
	@Mapping(target = "relationIdBytes", ignore = true)
	@Mapping(target = "statusBytes", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "vorgangIdBytes", ignore = true)
	@Mapping(target = "allFields", ignore = true)

	@Mapping(target = "bodyObj", source = "bodyObject")
	@Mapping(target = "orderString", source = "order")
	GrpcCommand toGrpc(Command vorgangCommand);

}