/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;

class VorgangAuthorizationServiceTest {

	@InjectMocks // NOSONAR
	private VorgangAuthorizationService service;

	@Nested
	class TestDoesOrganisationseinheitenIdMatch {

		private final Vorgang vorgang = VorgangTestFactory.create();

		@DisplayName("should return true if the filterByOrganisationseinheitenId is false")
		@Test
		void shouldReturnTrueOnNoFiltering() {
			var filter = FilterCriteriaTestFactory.createBuilder().filterByOrganisationseinheitenId(false).build();

			var isPermitted = callDoesOrganisationseinheitMatch(vorgang, filter);

			assertThat(isPermitted).isTrue();
		}

		@Nested
		class TestWithFilterByOrganisationsinheitenId {

			@DisplayName("should return false if the given user organisationseinheitenId list is empty")
			@Test
			void shouldReturnFalseOnFilteringWithEmptyList() {
				var filter = FilterCriteriaTestFactory.createBuilder().clearOrganisationseinheitIds().build();

				var isPermitted = callDoesOrganisationseinheitMatch(vorgang, filter);

				assertThat(isPermitted).isFalse();
			}

			@DisplayName("should return false if vorgang has no zustaendige stelle")
			@Test
			void shouldReturnFalseOnNonExistingZustaendigeStelle() {
				var vorgangWithoutZustaendigeStelle = VorgangTestFactory.createBuilder()
						.clearEingangs()
						.eingang(EingangTestFactory.createBuilder().zustaendigeStelle(null).build())
						.build();
				var isPermitted = callDoesOrganisationseinheitMatch(vorgangWithoutZustaendigeStelle, FilterCriteriaTestFactory.create());

				assertThat(isPermitted).isFalse();
			}

			@DisplayName("should return true if user organisationseinheitenId list contains organisationseinheitenId of vorgang")
			@Test
			void shouldReturnFalseOnNonMatch() {
				var filter = FilterCriteriaTestFactory.createBuilder().clearOrganisationseinheitIds().organisationseinheitId("NOT_MATCH").build();

				var isPermitted = callDoesOrganisationseinheitMatch(vorgang, filter);

				assertThat(isPermitted).isFalse();
			}

			@DisplayName("should return true if the users organisationseinheitenId list contains organisationseinheitenId from vorgang")
			@Test
			void shouldReturnTrueOnMatch() {
				var isPermitted = callDoesOrganisationseinheitMatch(vorgang, FilterCriteriaTestFactory.create());

				assertThat(isPermitted).isTrue();
			}
		}

		private boolean callDoesOrganisationseinheitMatch(Vorgang vorgang, FilterCriteria filterCriteria) {
			return service.authorizeByOrganisationseinheitenId(vorgang, filterCriteria);
		}
	}
}