/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang.redirect;

import static org.assertj.core.api.Assertions.*;

import java.util.Map;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoOperations;

import de.ozgcloud.common.test.DataITCase;
import de.ozgcloud.vorgang.vorgang.Vorgang;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;
import de.ozgcloud.vorgang.vorgang.redirect.Forwarding.Status;

@DataITCase
class ForwardingRepositoryITCase {

	@Autowired
	private ForwardingRepository repository;
	@Autowired
	private MongoOperations mongoOperations;

	private Vorgang vorgang;

	@BeforeEach
	void persistVorgang() {
		vorgang = mongoOperations.save(VorgangTestFactory.createBuilder().id(null).build());
	}

	@Test
	void saveForwarding() {
		var forwarding = ForwardingTestFactory.create();

		repository.save(vorgang.getId(), forwarding);

		var loadedVorgang = mongoOperations.findById(vorgang.getId(), Vorgang.class);
		assertThat(loadedVorgang.getForwardings()).hasSize(1);
		assertThat(loadedVorgang.getForwardings().get(0)).usingRecursiveComparison().isEqualTo(forwarding);
	}

	@Test
	void loadForwardings() {
		var forwarding = ForwardingTestFactory.create();
		repository.save(vorgang.getId(), forwarding);

		var loaded = repository.findByVorgangId(vorgang.getId());

		assertThat(loaded).hasSize(1);
		assertThat(loaded.get(0)).usingRecursiveComparison().isEqualTo(forwarding);
	}

	@Test
	void patch() {
		var forwarding = ForwardingTestFactory.createBuilder().id(UUID.randomUUID().toString()).build();
		repository.save(vorgang.getId(), forwarding);

		repository.patch(forwarding.getId(), Map.of(Forwarding.FIELD_STATUS, Forwarding.Status.FAILED));

		var loaded = repository.findByVorgangId(vorgang.getId());
		assertThat(loaded.get(0)).usingRecursiveComparison().ignoringFields("createdAt").isEqualTo(
				ForwardingTestFactory.createBuilder().id(forwarding.getId()).status(Status.FAILED).build());
	}
}
